function Size(width, height) {
	this.width = parseInt(width);
	this.height = parseInt(height);
	this.set = function (width, height) {
		this.width = width;
		this.height = height;
	};
	this.isValid = function () {
		return (this.width !== 0 && this.height !== 0);
	};
	this.toString = function () {
		return this.width + "x" + this.height;
	};
}

function Grid(cols, rows) {
	this.cols = cols;
	this.rows = rows;
	this.set = function (cols, rows) {
		this.cols = cols;
		this.rows = rows;
	};
	this.size = function() {
		return this.cols * this.rows;
	};
	this.toString = function () {
		return this.cols + "x" + this.rows;
	};
}

(function($) {
	$.widget("dvr.webView", {

		options: {
			connection: null,
			timeline: null,
			accelerated: false,
			maxHeight: undefined
		},

		selectedCameras: new Array(),
		rtspMode: false,
		cameraPanels: new Array(),
		layout: new Grid(2, 2),
		firstCamera: 0, // Kameranäkymän ensimmäinen kamera
		zoomedCamera: null, // Zoomatun kameran nimi. Tyhjä, jos ei zoomattu.
		viewMenu: null,
		rtspPort: 0,
		rtspKeepConnection: true,

		_create: function () {
			var that = this;
			var connection = this.options.connection;
		
			connection.onId = function (id) {
				var webViewImages = new WebViewImages(id);
				that.webViewImages = webViewImages;
				
                function updateCameraImage(uuid, imageSrc) {
					for (var i = 0; i < that.cameraPanels.length; ++i) {
						var panel = that.cameraPanels[i];
                        if (panel.camerapanel("option", "camera").uuid === uuid) {
							panel.camerapanel("update", imageSrc);
							break;
						}
					}
				};
			
                webViewImages.onCameraImage = function (uuid, image) {
                    updateCameraImage(uuid, (window.URL || window.webkitURL).createObjectURL(image));
				};
				
                webViewImages.onCameraImageBase64 = function (uuid, image) {
                    updateCameraImage(uuid, "data:image/jpeg;base64," + image);
				};
				
				webViewImages._onOpen = function () {
					this.setInterval(parseInt(that.viewMenu.kmenu("submenu", "interval").selected()));
					this.setQuality(parseInt(that.viewMenu.kmenu("submenu", "quality").selected()));
					for (var i = 0; i < that.cameraPanels.length; ++i) {
						var panel = that.cameraPanels[i];
						this.setSize(panel.camerapanel("width"), panel.camerapanel("height"));
						break;
					}
				};
			};

			connection.onMenu = function (xml) {
				var m = $($.parseXML(xml)).children("menu");

				$("#menubutton").click(function () { 
					if (that.rtspMode && that.options.accelerated) that.setRTSP(false, false);
					else that.viewMenu.kmenu("showHide");
				});

				var submenus = [];
				m.find("submenu").each(function () {
					var submenu = new SubMenu($(this).attr("id"), $(this).attr("title"), $(this).attr("multiselectable"), $(this).attr("icon"));
					$(this).children("menuitem").each( function () {
						var item;
						var value;
						var type;
						
						type = $(this).attr("type");
						if (type === "size") value = new Size($(this).attr("width"), $(this).attr("height"));
						else if (type === "grid") value = new Grid($(this).attr("rows"), $(this).attr("cols"));
                        else if (type === "camera") value = new Camera($(this).attr("title"), $(this).attr("uuid"));
						else value = $(this).attr("value");
						
						submenu.add(new MenuItem($(this).attr("title"), value, $(this).attr("selected")));
					});
					submenus.push(submenu);
				});

				var menuitems = [];
				m.children("menuitem").each(function () {
					menuitems.push(new MenuItem($(this).attr("title"), $(this).attr("value"), $(this).attr("selected"), $(this).attr("icon")));
				});

				that.viewMenu = $("#menu").kmenu({
					id: m.attr("id"),
					title: m.attr("title"),
					submenus: submenus,
					items: menuitems
				});

				var setCamerasId = 0;
				that.viewMenu.bind("kmenuchanged", function (event, submenu) {
					switch (submenu.id) {
					case "size":
						that.resizeCameraPanels();
						break;
					case "layout":
						that.setLayout(submenu.selected());
						break;
					case "interval":
						interval = parseInt(submenu.selected());
						if (that.webViewImages && that.webViewImages.isConnected()) that.webViewImages.setInterval(interval);
						break;
					case "quality":
						if (that.webViewImages && that.webViewImages.isConnected()) that.webViewImages.setQuality(submenu.selected());
						break;
					case "cameras":
						clearTimeout(setCamerasId);
						setCamerasId = setTimeout(function () {
							connection.setCameras(submenu.selected());
						}, 1000);
						break;
					case "info":
						connection.setShowStatus(submenu.selected().length === 1);
						break;
					case "mode":
						var mode = submenu.selected();
						if (mode === "jpegmode") that.setRTSP(false);
						else if (mode === "nonacceleratedrtsp") that.setRTSP(true, false);
						else if (mode === "acceleratedrtsp") that.setRTSP(true, true);
						that.viewMenu.kmenu("hide");
						break;
					}
				});

				that.viewMenu.bind("kmenuselected", function (event, item) {
					switch (item.value) {
					case "fullscreen": {
						if (document.msFullscreenElement || document.fullscreenElement || document.mozFullScreenElement || document.webkitFullscreenElement) {
							if (document.exitFullscreen) document.exitFullscreen();
							else if (document.webkitExitFullscreen) document.webkitExitFullscreen();
							else if (document.mozCancelFullScreen) document.mozCancelFullScreen();
							else if (document.msExitFullscreen) document.msExitFullscreen();
						} else {
							var i = $("body").get(0);
							if (i.requestFullscreen) i.requestFullscreen();
							else if (i.webkitRequestFullscreen) i.webkitRequestFullscreen();
							else if (i.mozRequestFullScreen) i.mozRequestFullScreen();
							else if (i.msRequestFullscreen) i.msRequestFullscreen();
						}
					}
					};
					that.viewMenu.kmenu("hide");
				});
				
				that.onReady();
			};
			
			connection.onLocalizations = function (xml) {
				var monthNames = [];
				var monthNamesShort = [];
				var dayNames = [];
				var dayNamesShort = [];
				var datetime = $(xml).find("datetime");

				$($(xml).find("months").children().get()).each(function () {
					monthNames.push($(this).attr("full"));
					monthNamesShort.push($(this).attr("short"));
				});

				$($(xml).find("days").children().get()).each(function () {
					dayNames.push($(this).attr("full"));
					dayNamesShort.push($(this).attr("short"));
				});   
				
				var localizations = $(xml).find("localizations");
				that.options.timeline.timeline("setDisabledText", localizations.attr("ocat_"));
				$("#datetime").datetimepicker({
					onSelect: function (datetimeText, datepicker) {
						that.options.timeline.timeline("setLive", false);
						connection.seekTo($(this).datepicker("getDate").getTime());
						if (that.webViewImages && !that.isRTSP()) that.webViewImages.getImages();
					},
					
					// Lokalisointi
					closeText: localizations.attr("done_"),
					prevText: localizations.attr("previous_"),
					nextText: localizations.attr("next_"),
					currentText: localizations.attr("live_"),
					monthNames: monthNames,
					monthNamesShort: monthNames,
					dayNames: dayNames,
					dayNamesShort: dayNamesShort,
					dayNamesMin: dayNamesShort,
					dateFormat: 'dd.mm.yy',
					firstDay: 1,
					isRTL: false,
					showMonthAfterYear: false,
					yearSuffix: '',
					showButtonPanel: false,
					minDate: new Date(2000, 0, 1, 0, 0),
					maxDate: new Date(2037, 11, 31, 23, 59)
				});
			};
			
			connection.start();
		}, // _create
		
		isRTSP: function () { return this.rtspMode; },
		
		setRTSP: function (state, accelerated) {
			var modeMenu = this.viewMenu.kmenu("submenu", "mode");
			if (state) {
				if (accelerated) modeMenu.select("acceleratedrtsp");
				else modeMenu.select("nonacceleratedrtsp");
			} else modeMenu.select("jpegmode");
			this.viewMenu.kmenu("update", modeMenu);
		
			this.firstCamera = 0;
			this.rtspMode = state;
			if (this.rtspMode) this.options.accelerated = accelerated;
			
			if (this.webViewImages && this.webViewImages.isConnected()) {
				if (this.rtspMode) this.webViewImages.setInterval(999999);
				else this.webViewImages.setInterval(parseInt(this.viewMenu.kmenu("submenu", "interval").selected()));
			}

			this.createCameraPanels();
		},
		
		resizeCameraPanels: function () {
			var s = this.toRealSize(this.viewMenu.kmenu("submenu", "size").selected());
			for (var i = 0; i < this.cameraPanels.length; ++i) {
				var panel = this.cameraPanels[i];
				// Tarkastetaan onko koko jo valmiiksi oikea
				if (i === 0 && panel.camerapanel("width") === s.width && panel.camerapanel("height") === s.height) return;
				panel.camerapanel("resize", s.width, s.height);
			}
			
			if (this.webViewImages && this.webViewImages.isConnected() && !this.isRTSP()) {
				this.webViewImages.setSize(s.width, s.height);
				this.webViewImages.getImages();
			}
		},

		createCameraPanelsId: 0,
		createCameraPanels: function (instant) {
			if (instant) {
				this._createCameraPanels();
				return;
			}

			// Ei tehdä kamerapaneeleja heti, koska tätä kutsutaan usein monta kertaa peräkkäin ja koska ainakin
			// vlc-pluginien jatkuva uudelleen luominen kaataa ie:n helposti.
			var that = this;
			clearTimeout(this.createCameraPanelsId);
			this.createCameraPanelsId = setTimeout(function () {
				that._createCameraPanels();
			}, 1000);
		},
		
		_createCameraPanels: function () {
			var that = this;
			var connection = this.options.connection;

			for (var i = 0; i < this.cameraPanels.length; ++i) this.cameraPanels[i].camerapanel("stop");
			this.cameraPanels = new Array();

			var s;
			if (this.isZoomMode()) {
				s = new Size(window.innerWidth, this.maxHeight());

				var div = $("<div></div>");
				this.element.html(div);
				var panel = div.camerapanel({
					camera: this.zoomedCamera,
					width: window.innerWidth,
					height: this.maxHeight(),
					rtspMode: this.rtspMode,
					rtspPort: this.rtspPort,
					viewId: connection.id,
					windowless: !this.options.accelerated
				});
				if (!this.rtspMode) panel.bind("camerapanelclick", function () { that.exitZoomMode(); });
				this.cameraPanels.push(panel);
			} else {
				var timeline = this.options.timeline;

				s = this.toRealSize(this.viewMenu.kmenu("submenu", "size").selected());
				var table = $("<table></table>").attr("cellspacing", 0).attr("cellpadding", 0).attr("border", 0);
				this.element.html(table);
				
				for (var row = 0; row < this.layout.rows; row++) {
					var tr = $("<tr></tr>");
					table.append(tr);
					for (var col = 0; col < this.layout.cols; col++) {
						var td = $("<td></td>");
						tr.append(td);

						var index = this.firstCamera + row * this.layout.cols + col;
                        var camera = (index >= this.selectedCameras.length) ? { label: "", uuid: "" }
							: this.selectedCameras[index];
						
						var panel = td.camerapanel({
							camera: camera,
							width: s.width,
							height: s.height,
							rtspMode: this.rtspMode,
							rtspPort: this.rtspPort,
							viewId: connection.id,
							windowless: !this.options.accelerated
						});
						
						if (!this.rtspMode && this.layout.size() > 1) {
							panel.bind("camerapanelclick", function (event, mouseDelta) {
								if(Math.sqrt(mouseDelta.x * mouseDelta.x + mouseDelta.y * mouseDelta.y) < 5) {
									var cam = $(this).camerapanel("option", "camera");
									that.isZoomMode() ? that.exitZoomMode() : that.enterZoomMode(cam);
								}
							});
						}
						panel.bind("camerapaneldrag", function (event, data) {
							if (!timeline.timeline("isLive")) return;
                            connection.panTilt($(this).camerapanel("option", "camera").uuid, data.x, data.y);
							if (that.webViewImages && that.webViewImages.isConnected() && !that.isRTSP()) {
								that.webViewImages.setSize(s.width, s.height);
								that.webViewImages.getImages();
							}
						});
						panel.bind("camerapaneldragend", function () {
							if (!timeline.timeline("isLive")) return;
                            connection.ptzStop($(this).camerapanel("option", "camera").uuid);
						});
						panel.bind("camerapanelzoom", function (event, data) {
							if (!timeline.timeline("isLive")) return;
                            connection.zoom($(this).camerapanel("option", "camera").uuid, data);
							if (that.webViewImages && that.webViewImages.isConnected() && !that.isRTSP()) {
								that.webViewImages.setSize(s.width, s.height);
								that.webViewImages.getImages();
							}
						});
						panel.bind("camerapanelzoomend", function (event, data) {
							if (!timeline.timeline("isLive")) return;
                            connection.ptzStop($(this).camerapanel("option", "camera").uuid);
						});
						
						this.cameraPanels.push(panel);
					}
				}
				
				
			}

			for (var i = 0; i < this.cameraPanels.length; ++i) this.cameraPanels[i].camerapanel("play");
			
			if (this.webViewImages && this.webViewImages.isConnected() && !this.isRTSP()) {
				this.webViewImages.setSize(s.width, s.height);
			}
						
			this.updateButtons();
			
			this._setActiveCameras();
		},
		
		isZoomMode: function () { return this.zoomedCamera !== null; },

		exitZoomMode: function () {
			this.zoomedCamera = null;
			this.createCameraPanels(true);
		},
		
		enterZoomMode: function (cam) {
			this.zoomedCamera = cam;
			this.createCameraPanels(true);
		},
		
		setLayout: function (layout_) {
			this.layout = layout_;
			this.firstCamera = 0;
			this.createCameraPanels();
		},
		
		_onFirstCameraChanged: function () {
			var rtspCommands = new Array();
			
			for (var i = 0; i < this.cameraPanels.length; ++i) {
				var panel = this.cameraPanels[i];

				var index = i + this.firstCamera;
				if (index >= this.selectedCameras.length) {
					if (this.isRTSP() && this.rtspKeepConnection) rtspCommands.push({command: "disable", stream: panel.camerapanel("uid")});
					else panel.camerapanel("clear");
				} else {
					if (this.isRTSP() && this.rtspKeepConnection) {
                        rtspCommands.push({command: "switch", stream: panel.camerapanel("uid"), camera: this.selectedCameras[index].uuid});
						panel.camerapanel("setCamera", this.selectedCameras[index]);
					} else {
						panel.camerapanel("setCamera", this.selectedCameras[index]);
						panel.camerapanel("play");
					}
				}
			}

			if (rtspCommands.length > 0) this.options.connection.setRTSPCommands(rtspCommands);
			
			this._setActiveCameras();
		},
		
		_setActiveCameras: function () {
			var activeCameras = new Array();
			for (var i = 0; i < this.cameraPanels.length; ++i) {
				var panel = this.cameraPanels[i];
                if (panel.camerapanel("isEnabled")) activeCameras.push(panel.camerapanel("option", "camera").uuid);
			}

			this.options.connection.setActiveCameras(activeCameras);
			if (this.webViewImages && this.webViewImages.isConnected()) {
				this.webViewImages.setActiveCameras(activeCameras);
				if (!this.isRTSP()) this.webViewImages.getImages();
			}
		},

		switchToNextCameras: function () {
			if (this.selectedCameras.length <= this.layout.size()) return;

			this.firstCamera += this.layout.size();
			if (this.firstCamera >= this.selectedCameras.length) this.firstCamera = 0;
			
			this._onFirstCameraChanged();
		},

		switchToPreviousCameras: function () {
			if (this.selectedCameras.length <= this.layout.size()) return;

			var newFirst = this.firstCamera - this.layout.size();
			if (newFirst < 0) {
				while (newFirst < this.selectedCameras.length - this.layout.size()) newFirst += this.layout.size();
			}
			this.firstCamera = newFirst;
			
			this._onFirstCameraChanged();
		},

		updateButtons: function () {
			if (this.isZoomMode() || this.layout.size() >= this.selectedCameras.length) { 
				// Ollaan zoom-tilassa tai layottiin mahtuu kaikki valitut kamerat => piilotetaan edellinen- ja seuraava-näppäimet
				$("#previousbutton").hide();
				$("#nextbutton").hide();
			} else {
				$("#previousbutton").show();
				$("#nextbutton").show();
			}
			
			var buttons = ["playbutton", "stopbutton", "livebutton", "rewindbutton", "playbackwardsbutton", "fastforwardbutton"];
			for (var i = 0; i < buttons.length; ++i) $("#" + buttons[i]).removeClass("button-state-active");

			function setActive(buttonId) { $("#" + buttonId).addClass("button-state-active"); }

			var s = this.options.timeline.timeline("playSpeed");
			if (this.options.timeline.timeline("isLive"))	setActive("livebutton");
			else if (s === 0)	setActive("stopbutton");
			else if (s === 1)	setActive("playbutton");
			else if (s === -1)	setActive("playbackwardsbutton");
			else if (s > 1)		setActive("fastforwardbutton");
			else				setActive("rewindbutton");
		},

		maxHeight: function() {
			return parseInt(this.options.maxHeight());
		},

		toRealSize: function (size) {
			// Tehdään deep copy size:stä, koska ei haluta muuttaa sitä
			var s = jQuery.extend(true, {}, size);
			var aspect = 16.0 / 9.0;

			// Jos size on epävalidi, asetetaan koot niin, että kuvat täyttää koko ikkunan
			if (!size || !s.isValid()) {
				var w = window.innerWidth / this.layout.cols;
				var h = this.maxHeight() / this.layout.rows;
				var ww = h * aspect;
				var hh = h;
				if(ww > w) {
					ww = w;
					h = ww / aspect;
				}
				s.set(Math.floor(ww), Math.floor(hh));
			}
			return s;
		},

		onReady: function () { // Kutsutaan, kun Menu on valmiina
			var that = this;
			var timeline = this.options.timeline;
			var connection = this.options.connection;
		
			// Aletaan kuunnella ilmoituksia (notifications.js)
			this.webnotifications = new WebNotifications();
			this.webnotifications.start();
			
			// Asetetaan viewin tapahtumienkäsittelijät
			var timelineRequested = false;
			connection.onTimeline = function (times) {
				timelineRequested = false;
				timeline.timeline("set", times.bounds, times.begin, times.end);
			};
			
			connection.onTimelineUpdate = function (times) {
				timeline.timeline("add", times.bounds, times.end);
			};
			
			connection.onZoomStarted = function (scale) {
				timeline.timeline("setScale", scale);
			};

			connection.onCameras = function (cameras) {
				that.selectedCameras = cameras;
				
				if (that.isZoomMode()) that.exitZoomMode();
				else that.createCameraPanels();
				timeline.timeline("setEnable", cameras.length > 0);
			};

			connection.onState = function (state, speed, unresolvedErrors, rtspPort) {
				if (rtspPort !== that.rtspPort) {
					that.rtspPort = rtspPort;
					for (var i = 0; i < that.cameraPanels.length; ++i) {
						var cameraPanel = that.cameraPanels[i];
						cameraPanel.camerapanel("setRTSPport", rtspPort);
						cameraPanel.camerapanel("stop");
						cameraPanel.camerapanel("play");
					}
				}
				
				var isLive = (speed === 2147483647);
				
				timeline.timeline("setError", unresolvedErrors);
				timeline.timeline("setState", state);
				timeline.timeline("setLive", isLive);
				timeline.timeline("setPlaySpeed", isLive ? 1 : speed);
				that.updateButtons();
				
				var shouldShowStatus = isLive && that.viewMenu.kmenu("submenu", "info").selected().length === 1;
				for (var i = 0; i < that.cameraPanels.length; ++i) that.cameraPanels[i].camerapanel("option", "mode", shouldShowStatus ? panelMode.STATUS : panelMode.SIMPLE);
			};
			
			connection.onAudioSpectrum = function (camera, audio) {
				for (var i = 0; i < that.cameraPanels.length; ++i) {
					var panel = that.cameraPanels[i];
                    if (panel.camerapanel("option", "camera").uuid === camera) {
						panel.camerapanel("updateAudioStatus", audio);
						break;
					}
				}
			};
			
			connection.onStatus = function (camera, status) {
				for (var i = 0; i < that.cameraPanels.length; ++i) {
					var panel = that.cameraPanels[i];
                    if (panel.camerapanel("option", "camera").uuid === camera) {
						panel.camerapanel("updateStatus", status);
						break;
					}
				}
			};
			
			connection.onTime = function (epoch) {
				timeline.timeline("setTime", epoch);
			};

			// Asetaan timelinen tapahtumankäsittelijät
			timeline.bind("timelinecreated", function () { connection.getTimeline(); });
			timeline.bind("timelinetimechanged", function (event, time) {
				connection.seekTo(time);
				if (that.webViewImages && !that.isRTSP()) that.webViewImages.getImages();
			});
			timeline.bind("timelineseekingstarted", function () { connection.setDragging(true); });
			timeline.bind("timelineseekingfinished", function () { connection.setDragging(false); connection.getTimeline(); });
			timeline.bind("timelinezoomed", function (event, count) { connection.zoomTimeline(count); });
			timeline.bind("timelineapproachingedge", function () {
				if (!timelineRequested) {
					timelineRequested = true;
					connection.getTimeline();
				}
			});
			timeline.bind("timelinedateset", function () {
				$("#datetime").datepicker("setDate", new Date(timeline.timeline("time") * 1000));
				if ($("#datetime").is(":visible")) $("#datetime").hide();
				else $("#datetime").show();
			});

			// Asetaan näppäinten tapahtumienkäsittelijät
			$("#previousbutton").click(function () { that.switchToPreviousCameras(); });
			$("#nextbutton").click(function () { that.switchToNextCameras(); });

			var mousedown = false;
			var seekSpeed = 100;
			function seekNext() {
				connection.stepForward();
				if (that.webViewImages && !that.isRTSP()) that.webViewImages.getImages();
				if (mousedown) setTimeout(seekNext, seekSpeed);
			};
			function seekPrevious() {
				connection.stepBackward();
				if (that.webViewImages && !that.isRTSP()) that.webViewImages.getImages();
				if (mousedown) setTimeout(seekPrevious, seekSpeed);
			};
			
			$("#stepbwbutton").mousedown(function () {
				mousedown = true;
				seekPrevious();
			}).bind("mouseup mouseleave", function () {
				mousedown = false;
			});
			$("#stepfwbutton").mousedown(function () {
				mousedown = true;
				seekNext();
			}).bind("mouseup mouseleave", function () {
				mousedown = false;
			});
			
			$(document).on("click", "#clipbutton", function () {
				var cameras = new Array();
                for (var i = 0; i < that.cameraPanels.length; ++i) cameras.push(that.cameraPanels[i].camerapanel("option", "camera").uuid);
				connection.createClip(cameras, parseInt(timeline.timeline("rangeBegin")), parseInt(timeline.timeline("rangeEnd"))); }
			);
			$(document).on("click", "#clearrangebutton", function () { timeline.timeline("clearRange"); } );
			$(document).on("click", "#rangebeginbutton", function () { timeline.timeline("setRangeBegin"); } );
			$(document).on("click", "#rangeendbutton", function () { timeline.timeline("setRangeEnd"); } );
			$(document).on("click", "#playbutton", function () { connection.play(); } );
			$(document).on("click", "#stopbutton", function () { connection.stop(); } );
			$(document).on("click", "#livebutton", function () { connection.live(); } );
			$(document).on("click", "#rewindbutton", function () { connection.rewind(); } );
			$(document).on("click", "#playbackwardsbutton", function () { connection.playBackwards(); } );
			$(document).on("click", "#fastforwardbutton", function () { connection.fastForward(); } );
			$(document).on("click", "#playpausebutton", function () { connection.togglePlayPause(); });

			var resizeId;
			$(window).resize(function() {
				clearTimeout(resizeId);
				resizeId = setTimeout(function () {
					that.resizeCameraPanels();
				}, 500);
			});

			$(document).keydown(function (e) {
				switch (e.keyCode) {
				case 37: // vasen
					if (!that.isZoomMode()) that.switchToPreviousCameras();
					break;
				case 38: break; // ylös
				case 39: // oikea
					if (!that.isZoomMode()) that.switchToNextCameras();
					break;
				case 40: break; // alas
				}
			});

			that.viewMenu.kmenu("load");
			connection.getState();
			connection.getTimeline();

			function switchLoop() {
				var timeout = that.viewMenu.kmenu("submenu", "timeout").selected();
				if (timeout > 0 && (timeline.timeline("isLive") || timeline.timeline("playSpeed") > 0) && !that.isZoomMode()) {
					that.switchToNextCameras();
					setTimeout(switchLoop, timeout * 1000);
				} else setTimeout(switchLoop, 2000);
			};
			switchLoop();

			if (this.webViewImages) this.webViewImages.start();
		} // onReady
	});

})(jQuery);
