var webconnection = new WebConnection();

const g_cellAspectRatio = 16.0 / 9.0;
const g_cellInvAspectRatio = 1.0 / g_cellAspectRatio;

var g_currentGrid = null;
var g_grids = [];

var g_canvases = [];
var g_resizeTimer = null;
var g_gridSocket = null;

//cfg
var g_hqStream = false;
var g_updateInterval = 200;
var g_showMenuBar = true;
var g_autoSelectGridIndex = 0;

const getUrlParameter = (param, defaultValue, type = 'string') => {
    const urlParams = new URLSearchParams(window.location.search);
    const value = urlParams.get(param);
    if(value == null) {
        return defaultValue;
    }
    switch (type) {
        case 'boolean':
            let lowerCase = value.toLowerCase();
            return (lowerCase === 'true') ? true : (lowerCase === 'false') ? false : defaultValue;
        case 'integer':
            const parsedInt = parseInt(value, 10);
            return !isNaN(parsedInt) ? parsedInt : defaultValue;
        default:
            return value ?? defaultValue;
    }
};

function drawTextBox(ctx, text, x, y) {
    const padding = 4;
    const fontSize = 14;
    ctx.font = `${fontSize}px Arial`; 
    const textWidth = ctx.measureText(text).width;
    const rectWidth = textWidth + padding * 2;
    const rectHeight = fontSize + padding * 2;
    ctx.fillStyle = '#1e1e1eb3';
    ctx.fillRect(x, y, rectWidth, rectHeight);
    ctx.fillStyle = '#f1f1f1e6';
    const textX = x + rectWidth/2 - textWidth/2;
    const textY = y + padding + (rectHeight - padding * 2 - fontSize) / 2;
    ctx.textBaseline = "top";
    ctx.fillText(text, textX, textY);
}

function drawCenteredTextBox(ctx, text, x, y) {
    const padding = 8;
    const fontSize = 14;
    ctx.font = `${fontSize}px Arial`; 
    const textWidth = ctx.measureText(text).width;
    const rectWidth = textWidth + padding * 2;
    const rectHeight = fontSize + padding * 2;

    x -= rectWidth / 2;
    y -= rectHeight / 2;

    ctx.fillStyle = '#1e1e1eb3';
    ctx.fillRect(x, y, rectWidth, rectHeight);
    ctx.fillStyle = '#f1f1f1e6';
    const textX = x + rectWidth/2 - textWidth/2;
    const textY = y + padding + (rectHeight - padding * 2 - fontSize) / 2;
    ctx.textBaseline = "top";
    ctx.fillText(text, textX, textY);
}

function requestPlay(gridIndex) {
    if(g_gridSocket == null) {
        return;
    }

    const navBar = document.getElementById('navBar');
    const containerHeight = window.innerHeight - navBar.clientHeight;
    const containerWidth = window.innerWidth;

    const playMsg = {
        command:"play",
        gridIndex: gridIndex,
        maxWidth: containerWidth,
        maxHeight: containerHeight,
        interval: g_updateInterval,
        highQuality: g_hqStream
    };
    g_gridSocket.send(JSON.stringify(playMsg));
}

function requestGrids() {
    if(g_gridSocket == null) {
        return;
    }
    const gridMsg = {
        command: "gridNames"
    };
    g_gridSocket.send(JSON.stringify(gridMsg));
}

function startPlay(gridIndex) {
    if(g_gridSocket == null) {
        return;
    }
    g_currentGrid = null;
    requestPlay(gridIndex);
}

function start() {
    const protocol = location.protocol === "https:" ? "wss://" : "ws://";
    const port = window.location.port.length == 0 ? "" : (":"+window.location.port);
    if(g_gridSocket) {
        g_gridSocket.close();
        g_gridSocket = null;
    }
    g_gridSocket = new WebSocket(protocol + window.location.hostname + port + "/webliveview");
    g_gridSocket.addEventListener("open", (event) =>{
        requestGrids();
    });
    g_gridSocket.addEventListener("closed", (event)=>{
        console.log("close")
    })
    g_gridSocket.addEventListener("error", (event)=>{
        console.log("error")
    })

    g_gridSocket.addEventListener("message", (event) =>{
        const message = JSON.parse(event.data);
        if(message.type == "error") {
            console.log(message.reason);
        } else if(message.type =="gridNames_error") {
            console.log(message.reason);
            const cameraGridcontainer = document.getElementById('cameraGridContainer');
            let centerText = document.createElement('div');
            centerText.innerHTML = message.reason;
            cameraGridcontainer.appendChild(centerText);
        } 
        else if(message.type == "gridNames") {
            fillGridsDropDown(message.grids);
            for(let i = 0; i < message.grids.length; i++) {
                const grid = message.grids[i];
                if(grid.gridIndex == g_autoSelectGridIndex) {
                    startPlay(grid.gridIndex);
                    break;
                }
            }
            if(g_autoSelectGridIndex == -1 && message.grids.length > 0) {
                startPlay(message.grids[0].gridIndex);
            }
        } else if(message.type == "gridInfo") {
            g_currentGrid = message;
            buildGrid(g_currentGrid.columns, g_currentGrid.rows);        
        } else if(message.type == "frames/jpg") {
            if(g_currentGrid == null) {
                console.log("no grid?");
                return;
            }

            if(message.gridIndex != g_currentGrid.gridIndex) {
                console.log("invalid grid?");
                return;
            }

            message.cells.forEach( (cell, index) =>{
                const canvas = g_canvases[index];
                if(cell.state) {
                    const err = cell.error;
                    if(err.length > 0) {
                        const label = cell.label;
                        loadErrorText(canvas, err, label);
                    } else {
                        const b64Image = cell.image;
                        const imageAspectRatio = cell.aspectRatio;
                        const flipx = cell.flipx;
                        const flipy = cell.flipy;

                        const validRotations = [0, 90, 180, 270];
                        const rotation = validRotations.includes(cell.rotation) ? cell.rotation : 0;
                        if(b64Image.length > 0) {
                            const label = cell.label;
                            loadB64Image(canvas, b64Image, imageAspectRatio, label,flipx, flipy, rotation);
                        }
                    }
                } else {
                    loadEmptyImage(canvas);
                }
            });
        }
    });
}

function fillGridsDropDown(grids) {
    g_grids = [];
    g_grids = grids
    let gridList =  document.getElementById("gridList");
    gridList.innerHTML = '';
    g_grids.forEach( (grid, index) => {
        let listItem = document.createElement('li');
        let link = document.createElement('a');
        link.classList.add("dropdown-item");
        link.href = "#";
        link.innerHTML = grid.name;
        link.setAttribute('data-grid-index', grid.gridIndex);
        listItem.appendChild(link);
        gridList.appendChild(listItem);
    });
}

function onGridListClicked(event) {
    if (event.target && event.target.classList.contains("dropdown-item")) {
        event.preventDefault();           
        const gridIndex = parseInt(event.target.dataset.gridIndex);
        startPlay(gridIndex);
    }
}

function loadEmptyImage(canvas) {
    const ctx = canvas.getContext('2d');
    ctx.clearRect(0,0, canvas.width, canvas.height);
}

function loadB64Image(canvas, b64Image, aspectRatio, cameraLabel, flipx, flipy, rotation) {
    const ctx = canvas.getContext('2d');
    if (canvas.tmpImage == null) {
        canvas.tmpImage = new Image();
    }
    const image = canvas.tmpImage;
    image.onload = function() {
        ctx.clearRect(0, 0, canvas.width, canvas.height);
        ctx.fillStyle = '#000000';
        ctx.fillRect(0, 0, canvas.width, canvas.height);

        let rotated = rotation === 90 || rotation === 270;
        let effectiveAspectRatio = rotated ? 1 / aspectRatio : aspectRatio;

        let drawWidth, drawHeight;
        const canvasRatio = canvas.width / canvas.height;
        if (effectiveAspectRatio < canvasRatio) {
            drawHeight = canvas.height;
            drawWidth = drawHeight * effectiveAspectRatio;
        } else {
            drawWidth = canvas.width;
            drawHeight = drawWidth / effectiveAspectRatio;
        }

        ctx.save();
        ctx.translate(canvas.width / 2, canvas.height / 2);
        ctx.rotate(rotation * Math.PI / 180);

        if (flipx) {
            ctx.scale(-1, 1);
        }
        if (flipy) {
            ctx.scale(1, -1);
        }

        if (rotated) {
            [drawWidth, drawHeight] = [drawHeight, drawWidth];
        }

        let x = -drawWidth / 2;
        let y = -drawHeight / 2;

        ctx.drawImage(image, x, y, drawWidth, drawHeight);
        ctx.restore();

        drawTextBox(ctx, cameraLabel, 5, 5);
    };
    image.onerror = function() {
        ctx.clearRect(0, 0, canvas.width, canvas.height);
        const centerX = canvas.width / 2;
        const centerY = canvas.height / 2;
        drawCenteredTextBox(ctx, "No image", centerX, centerY);
    };

    image.src = 'data:image/jpeg;base64,' + b64Image;
}

function loadErrorText(canvas, text, cameraLabel) {
    const ctx = canvas.getContext('2d');
    ctx.clearRect(0, 0, canvas.width, canvas.height);
    const centerX = canvas.width / 2;
    const centerY = canvas.height / 2;
    drawCenteredTextBox(ctx, text, centerX, centerY);
    drawTextBox(ctx, cameraLabel, 5, 5);
}

function calcCellSize(gridColumns, gridRows, cellGap, areaWidth, areaHeight) {
    const availableWidth = areaWidth - cellGap * (gridColumns - 1);
    const availableHeight = areaHeight - cellGap * (gridRows - 1);
    let cellWidth = availableWidth / gridColumns;
    let cellHeight = cellWidth * g_cellInvAspectRatio;
    let totalGridHeight = cellHeight * gridRows + cellGap * (gridRows - 1);
    if (totalGridHeight > availableHeight) {
        cellHeight = (availableHeight + cellGap) / gridRows - cellGap;
        cellWidth = cellHeight * g_cellAspectRatio;
    }
    return [cellWidth, cellHeight];
}

function buildGrid(gridColumns, gridRows) {
    const container = document.getElementById('cameraGridContainer');
    let cameraGrid = document.getElementById('cameraGrid');
    if(cameraGrid != null) {
        while(cameraGrid.firstChild) {
            cameraGrid.removeChild(cameraGrid.lastChild);
        }
        g_canvases = [];
    } else if(cameraGrid == null) {
        cameraGrid = document.createElement('div');
        cameraGrid.id = "cameraGrid";
        g_canvases = [];
    }

    const cellGap = 2;
    const [cellWidth, cellHeight] = calcCellSize(gridColumns, gridRows, cellGap, container.clientWidth,  container.clientHeight);
    const totalWidth = gridColumns * cellWidth + (gridColumns - 1) * cellGap;
    const totalHeight = cellHeight * gridRows + (gridRows - 1) * cellGap;

    cameraGrid.innerHTML = '';
    cameraGrid.style.width = `${totalWidth}px`;
    cameraGrid.style.height = `${totalHeight}px`;
    cameraGrid.style.display = 'grid';
    cameraGrid.style.gridTemplateColumns = `repeat(${gridColumns}, ${cellWidth}px)`;
    cameraGrid.style.gridAutoRows = `${cellHeight}px`;
    cameraGrid.style.gap = `${cellGap}px`;
    cameraGrid.style.marginLeft = 'auto';
    cameraGrid.style.marginRight = 'auto';

    for(let i = 0; i < gridRows * gridColumns; i++) {
        const cell = document.createElement('div');
        cell.style.width = '100%';
        cell.style.height = '100%';
        cell.style.backgroundColor = '#343434ff';
        cell.style.display = 'flex';
        cell.style.alignItems = 'center';
        cell.style.justifyContent = 'center';
        cell.style.borderRadius = '10px';

        let canvas = document.createElement('canvas');
        canvas.width = cellWidth;
        canvas.height = cellHeight;
        canvas.style.borderRadius = '10px';
        g_canvases.push(canvas);
        cell.appendChild(canvas);
        cameraGrid.appendChild(cell);
    }

    container.appendChild(cameraGrid);
}

function onWindowResize() {
    if(g_currentGrid == null) {
        return;
    }
    if(g_gridSocket == null) {
        return;
    }

    clearTimeout(g_resizeTimer);
    g_resizeTimer = setTimeout(function() {
        if(g_currentGrid == null) {
            return;
        }
        if(g_gridSocket.readyState != g_gridSocket.OPEN) {
            return;
        }
        startPlay(g_currentGrid.gridIndex);
    }, 500);
}

function logOut(e) {
    e.preventDefault();
    webconnection.logout();
}

function toSettings(e) {
    e.preventDefault();
    window.location.href = "/settings";
}

function onContentLoaded() {
    window.onresize = onWindowResize;
    window.onbeforeunload = function() {
        if(!g_gridSocket) {
            g_gridSocket.close();
        }
    }
    document.getElementById("logoutBtn").addEventListener("click", logOut);
    document.getElementById("settingsBtn").addEventListener("click", toSettings);
    document.getElementById("gridList").addEventListener("click", onGridListClicked);

    g_updateInterval = getUrlParameter('interval', 40, 'integer');
    g_hqStream = getUrlParameter('hq', false, 'boolean');
    g_showMenuBar = getUrlParameter('menuBar', true, 'boolean');
    g_autoSelectGridIndex = getUrlParameter('gridIndex', -1, 'integer');

    if(!g_showMenuBar) {
        const navBar = document.getElementById('navBar');
        navBar.style.display = 'none';
    }
    start();
}

document.addEventListener('DOMContentLoaded', onContentLoaded, false);
